<?php
/**
*
* Copyright (C) 2011 Consrcio Expresso Livre - 4Linux (www.4linux.com.br) e Prognus Software Livre (www.prognus.com.br)
*
* This program is free software; you can redistribute it and/or modify it under
* the terms of the GNU Affero General Public License version 3 as published by
* the Free Software Foundation with the addition of the following permission
* added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
* WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
* WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
*
* This program is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
* FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
* details.
*
* You should have received a copy of the GNU Affero General Public License
* along with this program; if not, see www.gnu.org/licenses or write to
* the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
* MA 02110-1301 USA.
*
* This code is based on the OpenXchange Connector and on the Prognus pSync
* Connector both developed by the community and licensed under the GPL
* version 2 or above as published by the Free Software Foundation.
*
* You can contact Prognus Software Livre headquarters at Av. Tancredo Neves,
* 6731, PTI, Bl. 05, Esp. 02, Sl. 10, Foz do Iguau - PR - Brasil or at
* e-mail address prognus@prognus.com.br.
*
* Servio de LDAP
*
* @package    Services
* @license    http://www.gnu.org/copyleft/gpl.html GPL
* @author     Consrcio Expresso Livre - 4Linux (www.4linux.com.br) e Prognus Software Livre (www.prognus.com.br)
* @version    1.0
* @sponsor    Caixa Econmica Federal
*/




/**
* Servio Ldap
*
* Classe responsvel pelas operaes com o LDAP
*
* @package    Service
* @license    http://www.gnu.org/copyleft/gpl.html GPL
* @author     Consrcio Expresso Livre - 4Linux (www.4linux.com.br) e Prognus Software Livre (www.prognus.com.br)
* @sponsor    Caixa Econmica Federal
* @version    1.0
*/
class LdapService
{
    var $limit = 11;

    var $allTargetTypes = array( 'i', 'g', 'l', 'u', 's' );

    var $connection;

    function LdapService()
    {
        
    }

	
	/**
     * Conecta com o LDAP
     *
     * @license    http://www.gnu.org/copyleft/gpl.html GPL
	 * @author     Consrcio Expresso Livre - 4Linux (www.4linux.com.br) e Prognus Software Livre (www.prognus.com.br)
	 * @sponsor    Caixa Econmica Federal
     * @param 	$host
     * @param 	$dn
     * @param 	$passwd
     * @param 	$ldapreferral
     */
    function connect($host='', $dn='', $passwd='', $ldapreferral=false)
    {
        if(!$host || $host == $GLOBALS['phpgw_info']['server']['ldap_host'])
        {
            $dn 	= $dn ? $dn : $GLOBALS['phpgw_info']['server']['ldap_root_dn'];
            $passwd = $passwd ? $passwd : $GLOBALS['phpgw_info']['server']['ldap_root_pw'];
            $host   = $host ? $host : $GLOBALS['phpgw_info']['server']['ldap_host'];
        }

        $conection = ldap_connect($host);

        if(!ldap_set_option($conection,LDAP_OPT_PROTOCOL_VERSION,3))
            $GLOBALS['phpgw_info']['server']['ldap_version3'] = False;
        
        ldap_set_option($conection, LDAP_OPT_REFERRALS, $ldapreferral);

        if($ldapreferral)
        {
            $GLOBALS['phpgw_info']['server']['user_ldap_referral'] = $dn;
            $GLOBALS['phpgw_info']['server']['password_ldap_referral'] = $passwd;
            ldap_set_rebind_proc($conection, ldap_rebind);
        }

        if($dn && $passwd && !@ldap_bind($conection,$dn,$passwd))
            @ldap_bind($conection,$dn,$passwd);

        $this->connection = $conection;

        return( $connection );
    }

    function _or( $toWrap )
    {
		if( !is_array( $toWrap ) )
			return( $toWrap );
	
		// 	if( count( $toWrap ) <= 1 )
		// 	    return implode( "", $toWrap );
	
		return $this->wrap( $toWrap, '|' );
    }

    function _and( $toWrap )
    {
		if( !is_array( $toWrap ) )
			return( $toWrap );
	
		// 	if( count( $toWrap ) <= 1 )
		// 	    return implode( "", $toWrap );
	
		return $this->wrap( $toWrap, '&' );
    }

    function _not( $toWrap )
    {
		return $this->wrap( $toWrap, '!' );
    }

    function wrap( $toWrap, $conditional = "" )
    {
		if( !is_array( $toWrap ) )
			$toWrap = array( $toWrap );
	
		$toWrap = array_unique( $toWrap );

		return "(".$conditional.implode( "", $toWrap ).")";
    }

	/**
     * Manipula o filtro de busca
     *
     * @license    http://www.gnu.org/copyleft/gpl.html GPL
	 * @author     Consrcio Expresso Livre - 4Linux (www.4linux.com.br) e Prognus Software Livre (www.prognus.com.br)
	 * @sponsor    Caixa Econmica Federal
     * @param 	$search
     * @param 	$targetTypes
     * @param 	$customFilter
     * @param 	$exact
     */
    function getSearchFilter( $search, $targetTypes = false, $customFilter = '', $exact = false )
    {
        $search = utf8_encode( $search );

		if( !$targetTypes )
				$targetTypes = $this->allTargetTypes;
	
		if( !is_array( $targetTypes ) )
			$targetTypes = array( $targetTypes );
	
		$searchFilter = '';
	
		foreach( $targetTypes as $targetType )
		{
			switch( $targetType )
			{
			case 'g':
			{
				//no caso de grupos, a busca so precisa ser feita em cima do CN
				$searchFilter = $this->stemFilter( $search, 'cn' );
			}
			break;
	
			default :
			{
			//parametros que a busca tem que ser sintaticas
				$searchFilter = /*array( */$this->stemFilter( $search, array(
							// UID e employeeNumber podem ser iguais ou muito similares, dependendo da organizacao
							'uid', /*'employeeNumber', */'cn',
							// givenName e SN sao complementares (nome e sobrenome)
							'givenName', 'sn', 'displayName' ) //),
							//parametros que a busca pode ser por aproximacao fonetica
						/*$this->approxFilter( $search, array( 
							// O CN e displayName geralmente sao a mesma coisa
							'cn', 'displayName' ))*/  );
	
				$searchFilter = $this->stemFilter( $search, array( 'cn', 'givenName', 'uid',
										'sn', 'displayName' ) );
			}
			break;
			}
		}

		$filter = array();
	
		if( $customFilter )
			$filter[] = $customFilter;
		if( $search )
			$filter[] = $searchFilter;
	
		return $this->_and( array(
			// Somente objetos relacionados com o Expresso
			$this->accountFilter( $targetTypes ),
			// Objetos ocultados e/ou desativados pelo Administrador nao podem ser exibidos nas consultas do Expresso
			$this->securityFilter( $targetTypes ),
			//foco da busca
			( $exact ? $this->_and( $filter ) : $this->_or( $filter ) )
			));
	
		// Verificoes extras para validar o resultado
		//  (&
		//     // Somente objetos com e-mail no formato da RFC822
		//    (mail=*@*)
		//  )
	}

	
	/**
     * Manipula o filtro de segurana
     *
     * @license    http://www.gnu.org/copyleft/gpl.html GPL
	 * @author     Consrcio Expresso Livre - 4Linux (www.4linux.com.br) e Prognus Software Livre (www.prognus.com.br)
	 * @sponsor    Caixa Econmica Federal
     * @param 	$targetTypes
     */
	function securityFilter( $targetTypes )
    {
		if( !$targetTypes )
				$targetTypes = $this->allTargetTypes;

		if( !is_array( $targetTypes ) )
			$targetTypes = array( $targetTypes );
	
		$typeFilter = array();
	
		foreach( $targetTypes as $targetType )
		{
			switch( $targetType )
			{
			case 'g': $typeFilter[] = "(objectClass=posixGroup)";
			break;
			
			default : $typeFilter[] = "(phpgwAccountStatus=A)";
			break;
			}
		}

		return $this->_and( array( '(!(phpgwAccountVisible=-1))', $this->_or( $typeFilter ) ) );
	}

	
	/**
     * Manipula o filtro da conta
     *
     * @license    http://www.gnu.org/copyleft/gpl.html GPL
	 * @author     Consrcio Expresso Livre - 4Linux (www.4linux.com.br) e Prognus Software Livre (www.prognus.com.br)
	 * @sponsor    Caixa Econmica Federal
     * @param 	$targetTypes
     */
    function accountFilter( $targetTypes )
    {
        if( !$targetTypes )
            $targetTypes = $this->allTargetTypes;

        if( !is_array( $targetTypes ) )
           $targetTypes = array( $targetTypes );

        $typeFilter = array();

        foreach( $targetTypes as $targetType )
            $typeFilter[] = '(phpgwAccountType='.$targetType.')';

        return $this->_and( array( '(objectClass=phpgwAccount)', $this->_or( $typeFilter ) ) );
    }

    
	
	/**
     * @license    http://www.gnu.org/copyleft/gpl.html GPL
	 * @author     Consrcio Expresso Livre - 4Linux (www.4linux.com.br) e Prognus Software Livre (www.prognus.com.br)
	 * @sponsor    Caixa Econmica Federal
     * @param 	$search
     * @param 	$params
     */
	function stemFilter( $search, $params )
    {
		$search = str_replace( ' ', '*', $search );
	
		if( !is_array( $params ) )
			$params = array( $params );
	
		foreach( $params as $i => $param )
			$params[$i] = "($param=*$search*)";
		
		return $this->_or( $params );
    }

    
	/**
     * Manipula o filtro fontico
     *
     * @license    http://www.gnu.org/copyleft/gpl.html GPL
	 * @author     Consrcio Expresso Livre - 4Linux (www.4linux.com.br) e Prognus Software Livre (www.prognus.com.br)
	 * @sponsor    Caixa Econmica Federal
     * @param 	$search
     * @param 	$params
     */
	function phoneticFilter( $search, $params )
    {
		if( eregi( "\d", $search ) )
			return( "" );
	
		if( !is_array( $params ) )
			$params = array( $params );
	
		foreach( $params as $i => $param )
			$params[$i] = "($param~=$search)";
		
		return $this->_or( $params );
    }

	
	
	
    function approxFilter( $search, $params )
    {
		return $this->_or( array( $this->stemFilter( $search, $params ),
				  $this->phoneticFilter( $search, $params ) ) );
    }

//     public function search()
//     {
// 	
//     }

    
	/**
     * Mtodo que realiza a busca no LDAP
     *
     * @license    http://www.gnu.org/copyleft/gpl.html GPL
	 * @author     Consrcio Expresso Livre - 4Linux (www.4linux.com.br) e Prognus Software Livre (www.prognus.com.br)
	 * @sponsor    Caixa Econmica Federal
     * @param 	$search
     * @param 	$justthese
     * @param 	$context
     * @param 	$accountType
     * @param 	$sort
     */
	public function accountSearch($search, $justthese = "*", $context = false , $accountType = false, $sort = false)
    {
         if( !$this->connection )
            $this->connect();

         $filter =  $this->getSearchFilter($search,$accountType);

         if( !$context )
             $context = $GLOBALS['phpgw_info']['server']['ldap_context'];

         $ls = ldap_search( $this->connection, utf8_encode($context), $filter, $justthese, 0, $this->limit );

         if($sort)
             ldap_sort( $this->connection, $ls, $sort );

         $entries = ldap_get_entries( $this->connection, $ls );

         if( !$entries ) return( null );

         $return = array();

         for ($i=0; $i < $entries["count"]; $i++)
         {
               $entrieTmp = array();
               foreach ($entries[$i] as $index => $value)
               {
                   if(!is_numeric($index) && $index != 'count')
                   {
                       if(is_array($value))
                       {
                           if(count($value) == 2)
                               $entrieTmp[$index] = utf8_decode($value['0']);
                           else
                           {
                               foreach ($value as $index2 =>$value2)
                               {
                                    if($index != 'count')
                                        $entrieTmp[$index][$index2] = utf8_decode($value2);
                               }
                           }
                       }
                       else
                           $entrieTmp[$index] = utf8_decode($value);
                   }
               }

               $return[] = $entrieTmp;
         }

         return( $return );
    }

}

ServiceLocator::register( 'ldap', new LdapService() );

